#===========================================
# Simple Trick 0.8
#===========================================
module FeatureModifier
	DEBUG_MODE = true
	# Modified actor check becomes simpler
	def self.should_modify?(actor)
		MODIFICATIONS.key?(actor.id)
	end
	# Add to FeatureModifier module (around line 20)
	def self.modify_complex_feature(actor, code, data_id, new_value)
		feature = find_feature(actor, code, data_id)
		return unless feature
		# Special handling for hash/array values
		case feature.value
		when Hash
			log "  Modifying hash-based feature"
			# Deep merge for hash values (common in skill changing features)
			new_value.each do |k, v|
				feature.value[k] = v
				log "    Updated key #{k} => #{v}"
			end
		when Array
			log "  Modifying array-based feature"
			# For arrays, we can either replace or append
			if new_value.is_a?(Array)
				feature.value.replace(new_value)
			else
				feature.value << new_value
			end
		else
			# Standard scalar value replacement
			feature.value = new_value
		end
	end
	# Finds a specific feature on an actor
	# Works with modified features(code) interface we discovered through debugging
	def self.find_feature(actor, code, data_id)
		log "\n=== Searching for feature: code #{code}, data_id #{data_id} on #{actor.name} ==="
		# Use the confirmed features(code) interface instead of vanilla array access
		# This was crucial - we found through debugging that features expects a code parameter
		features = actor.features(code) rescue []
		log "  Found #{features.size} features with code #{code}"
		# Only log our target feature if found (reduces debug spam)
		features.find { |f| f.data_id == data_id }.tap do |found|
			if found
				log "  FOUND target feature"
				log "  Feature details: #{found.inspect}"
			else
				log "  Target feature not found"
			end
		end
	end
	# Main modification method - applies all configured changes to an actor
	def self.modify_actor_features(actor)
		return unless should_modify?(actor)
		# Support both old and new MODIFICATIONS format
		modifications = MODIFICATIONS[actor.id]
		if modifications.is_a?(Array)
			# New array-based format
			process_modification_array(actor, modifications)
		else
			# Original hash-based format (backward compatible)
			log "\n===== MODIFYING ACTOR #{actor.id} (#{actor.name}) ====="
			modifications.each do |code, data_mods|
				# Small optimization: get features once per code type
				features = actor.features(code) rescue []
				log "Processing features of type #{code}"
				data_mods.each do |data_id, value|
					log "Processing feature #{code}-#{data_id} -> #{value}"
					# Directly modify the found feature object
					# This works because we're editing the actual reference stored in the actor's data
					if feature = features.find { |f| f.data_id == data_id }
						track_modification(actor, code, data_id, feature.value, value)
						if value.is_a?(Hash) || value.is_a?(Array)
							modify_complex_feature(actor, code, data_id, value)
						else
							feature.value = value
							log "  Modified #{code}-#{data_id} = #{value}"
						end
					else
						# If feature doesn't exist, inject it directly into the actor's features
						# We use instance_variable_get to bypass any potential method overrides
						add_feature(actor, code, data_id, value)
					end
				end
			end
		end
	end
	# Add these new methods while keeping old ones
	def self.process_modification(actor, code, data_id, value, merge=false)
		features = actor.features(code) rescue []
		feature = features.find { |f| f.data_id == data_id }

		if feature
			if merge && value.is_a?(Hash)
				feature.value.merge!(value)
			elsif value.is_a?(Hash) || value.is_a?(Array)
				modify_complex_feature(actor, code, data_id, value)
			else
				feature.value = value
			end
			track_modification(actor, code, data_id, feature.value, value)
		else
			add_feature(actor, code, data_id, value, merge: merge)
		end
	end
	# Adds a new feature to an actor
	# Uses direct instance variable access to ensure it persists
	def self.add_feature(actor, code, data_id, value, merge=false)
		log "  Creating and adding new feature #{code}-#{data_id}"
		# Special initialization for complex features
		# (Preserving our discovery that some systems need special handling)
		new_feature = create_feature_object(code, data_id, value)
		# Critical insights preserved from our debugging:
		# 1. Using instance_variable_get(:@features) bypasses overridden methods/caching
		# 2. Some scripts may store features differently, so we need fallbacks
		features_array = get_true_features_array(actor)
		# Our discovery that some scripts might have nil features array
		features_array ||= []
		# Add merge-specific handling
		if merge && (value.is_a?(Hash) || value.is_a?(Array))
			if existing = features_array.find { |f| f.code == code && f.data_id == data_id }
				log "  Merging into existing feature"
				modify_complex_feature(actor, code, data_id, value)
				return
			end
			# Initialize with empty container for merging
			new_feature.instance_variable_set(:@value, value.is_a?(Hash) ? {} : [])
		end
		# Check if feature exists (preserve our optimization to avoid duplicates)
		unless features_array.any? { |f| f.code == code && f.data_id == data_id }
			features_array << new_feature
			log "  Added successfully: #{find_feature(actor, code, data_id) ? 'Verified' : 'Failed!'}"
			# New insight from testing: Some systems need refresh after modification
			actor.refresh if actor.respond_to?(:refresh)
		else
			log "  Feature already exists - modifying instead"
			modify_complex_feature(actor, code, data_id, value)
		end
	end

	# Add these new methods while keeping all existing code intact
	def self.process_single_modification(actor, mod)
		# Special handling for skill cost features

		# Our discovery: get features once per code type for performance
		features = actor.features(mod[:code]) rescue []
		log "Processing #{mod[:code]}-#{mod[:data_id]}"
		# Find existing feature using our proven method
		feature = features.find { |f| f.data_id == mod[:data_id] }
		if feature
			# Preserve our modification tracking
			track_modification(actor, mod[:code], mod[:data_id], feature.value, mod[:value])
			# Keep our special handling for complex types
			if mod[:merge] == :deep && (mod[:value].is_a?(Hash) || mod[:value].is_a?(Array))
				modify_complex_feature(actor, mod[:code], mod[:data_id], mod[:value])
			else
				feature.value = mod[:value]
				log "  Modified #{mod[:code]}-#{mod[:data_id]} = #{mod[:value]}"
			end
		else
			# Use our existing add_feature but with new merge parameter
			add_feature(actor, mod[:code], mod[:data_id], mod[:value], mod[:merge] || false)
		end
	end

	# New method to handle the array-based MODIFICATIONS
	def self.process_modification_array(actor, modifications)
		modifications.each do |mod|
			# Preserve our debugging logs
			log "\n===== MODIFYING ACTOR #{actor.id} (#{actor.name}) =====" if mod == modifications.first
			process_single_modification(actor, mod)
		end
	end

		def self.get_true_features_array(actor)
		unless actor.is_a?(Game_Actor)
			log "CRITICAL: Invalid actor passed to get_true_features_array: #{actor.inspect}"
			return []
		end

		begin
			# Try instance variable first
			if actor.instance_variable_defined?(:@features)
				features = actor.instance_variable_get(:@features)
				return features if features.is_a?(Array)
			end

			# Handle parameterized features method
			if actor.respond_to?(:features)
				method = actor.method(:features)
				case method.arity
				when 0
					return actor.features
				when 1 # If features requires a code parameter
					# Return a proxy array that will collect all features
					return FeatureProxyArray.new(actor)
				else
					log "Unsupported features method arity: #{method.arity}"
				end
			end

			# Final fallbacks
			actor.respond_to?(:_features) ? actor._features : []
		rescue => e
			log "ERROR in get_true_features_array: #{e.message}"
			[]
		end
		def self.modify_skill_cost_feature(actor, data_id, new_value)
			features = actor.features(68) || []
			feature = features.find { |f| f.data_id == data_id }

			if feature
				if feature.value.is_a?(Array)
					# If it's already an array, append the new entry
					feature.value << new_value
				else
					# Convert existing single entry to array
					feature.value = [feature.value, new_value]
				end
			else
				# Create new feature with array value
				add_feature(actor, 68, data_id, [new_value])
			end
		end
	end

	# Proxy array to handle parameterized features methods
	class FeatureProxyArray < Array
		def initialize(actor)
			@actor = actor
			super()
		end

		def <<(feature)
			# When adding a feature, we need to handle it specially
			if @actor.instance_variable_defined?(:@features)
				@actor.instance_variable_get(:@features) << feature
			else
				# If we can't find a direct features array, try to add through official channels
				if @actor.respond_to?(:add_feature)
					@actor.add_feature(feature)
				else
					super
				end
			end
		end
	end

	# Creates appropriate feature object based on type
	# (Documents our findings about special feature types)
	def self.create_feature_object(code, data_id, value)
		# Our discovery that hash/array values need special handling
		if value.is_a?(Hash) || value.is_a?(Array)
			feature = RPG::BaseItem::Feature.new(code, data_id, 0)
			# Critical finding: Some scripts modify value= setter, so we bypass it
			feature.instance_variable_set(:@value, value.dup)
			# Learned from testing: Some scripts check for this flag
			feature.instance_variable_set(:@dynamic_add, true)
			feature
		else
			# Standard feature creation path we confirmed works
			RPG::BaseItem::Feature.new(code, data_id, value)
		end
	end
	# Cleaner logging that only shows essential information
	def self.log(msg)
		return unless FeatureModifier::DEBUG_MODE
		# Remove redundant indentation from logs
		p "[FeatureMod] #{msg.gsub(/^\s{2,}/, '')}"
	end
	@modified_features ||= {}
	def self.track_modification(actor, code, data_id, old_val, new_val)
		@modified_features[actor.id] ||= {}
		@modified_features[actor.id][[code, data_id]] = {
			original: old_val,
			modified: new_val,
			timestamp: Time.now
		}
	end
end
# Installation via aliased methods
class Game_Actor
	# Hook into actor initialization
	# This ensures our modifications apply when actors are first created
	alias featmod_init initialize
	def initialize(actor_id)
		featmod_init(actor_id)
		FeatureModifier.modify_actor_features(self)
	end
end
class Scene_Load
	# Re-apply modifications after loading saves
	# Necessary because some systems reinitialize actors on load
	alias featmod_load on_load_success
	def on_load_success
		featmod_load
		$game_party.all_members.each { |a| FeatureModifier.modify_actor_features(a) }
	end
end
# rpg maker vx ace uses Ruby 1.9.3

# Cheeky ass solution for ONE feature, god damn it.
class Game_Battler < Game_BattlerBase
	alias original_skills_tp_cost skill_tp_cost
	def skill_tp_cost(skill)
		original_cost = original_skills_tp_cost(skill)
		return original_cost unless actor?

		if (multiplier = FeatureModifier::TPCONFIG_SETTINGS.dig(self.id, skill.id))
			(original_cost * multiplier).ceil
		else
			original_cost
		end
	end
end


# Put this in your Patch folder
# Original method with ONE LINE changed (just the config source)
# Direct patch to skill application - NO TARGET GUESSING
class Game_Battler
  # Store original method reference first
  alias fm_item_apply item_apply unless method_defined?(:fm_item_apply_204882)

  def item_apply(user, item,*args)
    # Preserve original behavior
    success = fm_item_apply(user, item, *args)

    # Only process if successful skill use
    if success && item.is_a?(RPG::Skill)
      # Safely navigate hash without &. operator
      user_config = FeatureModifier::SKILLSTATER[user.id]
      skill_config = user_config && user_config[item.id]

      if skill_config
        # Process target effects
        if skill_config[:target] && self != user # Prevent double application
          skill_config[:target].each do |state_id, chance|
            next unless chance > rand(100)
            add_state(state_id)
            # Debug output
            p "Applied state #{state_id} to #{self.name}" if $TEST
          end
        end

        # Process self effects
        if skill_config[:self]
          skill_config[:self].each do |state_id, chance|
            next unless chance > rand(100)
            user.add_state(state_id)
            p "Applied self state #{state_id} to #{user.name}" if $TEST
          end
        end
      end
    end

    success
  end
end



